'use strict';

exports.__esModule = true;

var _reduxThunk = require('redux-thunk');

var _reduxThunk2 = _interopRequireDefault(_reduxThunk);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function createRealThunks(logic, input, dispatch, getState) {
  var actions = {};

  Object.keys(logic.actions).forEach(function (actionKey) {
    actions[actionKey] = function () {
      var _logic$actions;

      return dispatch((_logic$actions = logic.actions)[actionKey].apply(_logic$actions, arguments));
    };
    actions[actionKey].toString = logic.actions[actionKey].toString;
  });

  var get = function get(key) {
    return key ? logic.selectors[key](getState()) : logic.selector(getState());
  };
  var fetch = function fetch() {
    var results = {};

    var keys = Array.isArray(arguments[0]) ? arguments[0] : arguments;

    for (var i = 0; i < keys.length; i++) {
      results[keys[i]] = get(keys[i]);
    }

    return results;
  };

  return input.thunks(Object.assign({}, logic, { actions: actions, dispatch: dispatch, getState: getState, get: get, fetch: fetch }));
}

exports.default = {
  name: 'thunk',

  beforeReduxStore: function beforeReduxStore(options) {
    options.middleware.push(_reduxThunk2.default);
  },
  afterCreate: function afterCreate(logic, input) {
    if (!input.thunks) {
      return;
    }

    var realThunks = void 0;
    var thunkKeys = Object.keys(input.thunks(logic));
    var thunkFunctions = {};

    thunkKeys.forEach(function (thunkKey) {
      thunkFunctions[thunkKey] = function () {
        for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
          args[_key] = arguments[_key];
        }

        return function (dispatch, getState) {
          var _realThunks;

          if (!realThunks) {
            realThunks = createRealThunks(logic, input, dispatch, getState);
          }
          return (_realThunks = realThunks)[thunkKey].apply(_realThunks, args);
        };
      };
    });

    logic.actions = Object.assign({}, logic.actions, thunkFunctions);
  }
};